/* 
 * FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
 * Copyright (C) 2005-2012, Anthony Minessale II <anthm@freeswitch.org>
 *
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
 *
 * The Initial Developer of the Original Code is
 * Anthony Minessale II <anthm@freeswitch.org>
 * Portions created by the Initial Developer are Copyright (C)
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * Ken Rice <krice@freeswitch.org>
 * 
 * mod_diydp.c -- DIY Dialplan Demo Module
 * Includes Dialplan, API, and a Dialplan Application Example
 * 
 */


#include <switch.h>

SWITCH_MODULE_LOAD_FUNCTION(mod_diydp_load);
SWITCH_MODULE_SHUTDOWN_FUNCTION(mod_diydp_shutdown);
SWITCH_MODULE_DEFINITION(mod_diydp, mod_diydp_load, mod_diydp_shutdown, NULL);

#define DEFAULT_USER "999999"
#define DEFAULT_OUTBOUND_PROFILE "default"
#define DEFAULT_LCR_DB_HOST "db1"
#define DEFAULT_LCR_DB_NAME "test_db"
#define DEFAULT_LCR_DB_USER "user"
#define DEFAULT_LCR_DB_PASSWORD "password"
#define DB_QUEUE_LEN 200000

static struct {
        char *db_host;
        char *db_name;
        char *db_user;
        char *db_password;
        char *db_options;
        char *default_user;
        char *outbound_profile;
	int id_pool;
	switch_queue_t *queue;
        switch_memory_pool_t *pool;
	switch_mutex_t *mutex;
} globals;

SWITCH_DECLARE_GLOBAL_STRING_FUNC(set_global_db_host, globals.db_host);
SWITCH_DECLARE_GLOBAL_STRING_FUNC(set_global_db_name, globals.db_name);
SWITCH_DECLARE_GLOBAL_STRING_FUNC(set_global_db_user, globals.db_user);
SWITCH_DECLARE_GLOBAL_STRING_FUNC(set_global_db_password, globals.db_password);
SWITCH_DECLARE_GLOBAL_STRING_FUNC(set_global_db_options, globals.db_options);
SWITCH_DECLARE_GLOBAL_STRING_FUNC(set_global_default_user, globals.default_user);
SWITCH_DECLARE_GLOBAL_STRING_FUNC(set_global_outbound_profile, globals.outbound_profile);

/**
 * Configuration module from configuration file
 */
static switch_status_t load_config(void)
{

	static char *FMT = "host=%s dbname=%s user=%s password=%s";

	char *cf = "diydp.conf";
	char *dboptions = NULL;

	switch_xml_t cfg, xml = NULL, settings;
	switch_status_t status = SWITCH_STATUS_SUCCESS;
	
	int len;
	
	set_global_db_host(DEFAULT_LCR_DB_HOST);
	set_global_db_name(DEFAULT_LCR_DB_NAME);
	set_global_db_user(DEFAULT_LCR_DB_USER);
	set_global_db_password(DEFAULT_LCR_DB_PASSWORD);
	set_global_default_user(DEFAULT_USER);
	set_global_outbound_profile(DEFAULT_OUTBOUND_PROFILE);

	if (!(xml = switch_xml_open_cfg(cf, &cfg, NULL))) {
		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Cannot Open Modules Config [%s]\n", cf);
		status = SWITCH_STATUS_FALSE;
		goto done;
	}

	if ((settings = switch_xml_child(cfg, "settings"))) {
		switch_xml_t param;
		for (param = switch_xml_child(settings, "param"); param; param = param->next) {
			const char *name = switch_xml_attr_soft(param, "name");
			const char *value = switch_xml_attr_soft(param, "value");
			if (!strcasecmp("default-user", name)) {
				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "Setting default user to %s\n", value);
				set_global_default_user(value);
			} else if (!strcasecmp("db-host", name)) {
				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "Setting LCR db host to %s\n", value);
				set_global_db_host(value);
			} else if (!strcasecmp("db-user", name)) {
				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "Setting LCR db user to %s\n", value);
				set_global_db_user(value);
			} else if (!strcasecmp("db-password", name)) {
				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "Setting LCR db password\n");
				set_global_db_password(value);
			} else if (!strcasecmp("db-name", name)) {
				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "Setting LCR db name to %s\n", value);
				set_global_db_name(value);
			} else if (!strcasecmp("default-profile", name)) {
				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "Setting default profile to %s\n", value);
				set_global_outbound_profile(value);
			} else {
				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_WARNING, "ignoring unknown param: %s = %s\n", name, value);
			}
		}
	}

        len = strlen(globals.db_host) + strlen(globals.db_name) + strlen(globals.db_user) + strlen(globals.db_password) + strlen(FMT);
        if (len > 1024) {
                return 0;
        }
        dboptions = (char *) malloc(len);
        sprintf(dboptions, FMT, globals.db_host, globals.db_name, globals.db_user, globals.db_password);
	set_global_db_options(dboptions);
	free(dboptions);

  done:
	if (xml) {
		switch_xml_free(xml);
	}

	return status;
}

#define DEMO1_APP "demo1"
#define DEMO1_APP_SYNTAX "<arg1> [<arg2>]"
/* static void lcr_adv_app_function(switch_core_session_t *session, char *data) */
SWITCH_STANDARD_APP(demo1_app_function)
{
	int argc = 0;
	char *argv[4] = { 0 };
	char *mydata = NULL;
	
	switch_channel_t *channel = switch_core_session_get_channel(session);

	assert(channel != NULL);

	if (!(mydata = switch_core_session_strdup(session, data))) {
		return;
	}

	if ((argc = switch_separate_string(mydata, ' ', argv, (sizeof(argv) / sizeof(argv[0]))))) {
		if (argc < 1) {
			switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "This Dialplan App Takes atleast 1 argument\n");
		} else {

			switch_channel_set_variable(channel, "demo_app_var1", argv[0]);
			switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_INFO, "set demo_app_var1 = %s\n", argv[0]);

			if (argc > 1) {
				switch_channel_set_variable(channel, "demo_app_var2", argv[1]);
				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_INFO, "set demo_app_var2 = %s\n", argv[1]);
			}
		}
	}
}

SWITCH_STANDARD_DIALPLAN(demo1_dialplan_hunt)
{

	switch_caller_extension_t *extension = NULL;
	switch_channel_t *channel = switch_core_session_get_channel(session);
	char *account = NULL, *dest = NULL;
	const char *client_host = NULL;
	char *ptr = NULL;

	/* Make sure we have a caller Profile. THis is where we get all the good information for the call
	 * Defined by struct switch_caller_profile in include/switch_caller.h                             */
	if (!caller_profile) {
		caller_profile = switch_channel_get_caller_profile(channel);
	}

	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "Route Lookup on [%s] from [%s]\n", caller_profile->destination_number,
								caller_profile->network_addr);

	/* Check to see if we have a channel variable we want to do something with */
	if ((client_host = switch_channel_get_variable(channel, "sip_h_X-SourceAddr"))){
		if (zstr(client_host)) {
			client_host = caller_profile->network_addr;
		}
		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "Got Client Behind OpenSIPS on [%s] from [%s]\n", client_host, 
			caller_profile->network_addr);
	}

	switch_channel_set_variable(channel, "source_addr", client_host);

	/*
	 * Do an IP Lookup Here from your source. 
	 * if the following is True then we will Force a 403 or other response via the respond application
	 */
	if (1 == 0){
		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "Host Not Authorized [%s]\n", client_host);
		/* Generate an Extesnion */
		if ((extension = switch_caller_extension_new(session, caller_profile->destination_number, caller_profile->destination_number)) == 0) {
			switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "memory error!\n");
			return NULL;
		}
		/* Add An Application */
		switch_caller_extension_add_application(session, extension, "respond", "403 Forbidden ");
		return extension;
	}

	/* If  customer sends an account code lets split it off the front of number, delimited by a # sign 
	 * example destination: sip:123456#18005551212@mydomain.com            */
	dest = switch_core_session_strdup(session, caller_profile->destination_number);
	if ((ptr = strchr(caller_profile->destination_number, '#'))) {
		account = dest;
		dest = switch_core_session_strdup(session, ptr + 1);
		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "INCOMING ACCOUNTCODE [%s]\n", account);
	}

	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "DEST AFTER ACCOUNT FOR ACCOUNTCODE [%s]\n", dest);

	/* set a couple of channel variables for giggles */
	switch_channel_set_variable(channel, "destnum", dest);
	switch_channel_set_variable(channel, "rdnis", caller_profile->destination_number);
	switch_channel_set_variable(channel, "accountcode", account);
	switch_channel_set_variable(channel, "set", "continue_on_fail=TRUE");
	switch_channel_set_variable(channel, "set", "hangup_after_bridge=TRUE");

	if ((extension = switch_caller_extension_new(session, caller_profile->destination_number, caller_profile->destination_number)) == 0) {
		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "memory error!\n");
		return NULL;
	}

	switch_channel_set_variable(channel, SWITCH_HANGUP_AFTER_BRIDGE_VARIABLE, "true");

	switch_caller_extension_add_application(session, extension, "info", "");

		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "INCOMING Add Bridge to Toll Free \n");
	switch_caller_extension_add_application(session, extension, "bridge", "sofia/internal/5551212@sip.tollfreegateway.com");

		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "INCOMING Add Bridge to 1616 conf \n");
	switch_caller_extension_add_application(session, extension, "bridge", "sofia/internal/1616@conference.freeswitch.org");

	return extension;
}

#define DEMO2_API "demo2"
#define DEMO2_API_SYNTAX "<arg1> [arg2]"
SWITCH_STANDARD_API(demo2_api_function)
{
	int argc = 0;
	char *argv[4] = { 0 };
	char *mydata = NULL;

	if (session) {
		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "This function cannot be called from the dialplan.\n");
		return SWITCH_STATUS_FALSE;
	}

	if (zstr(cmd)) {
		stream->write_function(stream, "Usage: " DEMO2_API " " DEMO2_API_SYNTAX "\n");
		return SWITCH_STATUS_SUCCESS;
	}

	if (!(mydata = strdup(cmd))) {
		stream->write_function(stream, "Memory error!\n");
		return SWITCH_STATUS_SUCCESS;
	}

	if ((argc = switch_separate_string(mydata, ' ', argv, (sizeof(argv) / sizeof(argv[0]))))) {
		if (argc == 0 || argc > 2) {
			stream->write_function(stream, DEMO2_API " takes 1 or 2 args.\n");
			goto done;
		}
		
		stream->write_function(stream, "Argument 1: %s\n", argv[0]);
		if (argc == 2) {
			stream->write_function(stream, "Argument 2: %s\n", argv[1]);
		}
	}

 done:
	switch_safe_free(mydata);

	return SWITCH_STATUS_SUCCESS;
}

/* SWITCH_MOD_DECLARE(switch_status_t) switch_module_load(switch_loadable_module_interface_t **module_interface, char *filename) */
SWITCH_MODULE_LOAD_FUNCTION(mod_diydp_load)
{
	switch_api_interface_t *api_interface;
	switch_application_interface_t *app_interface;
	switch_dialplan_interface_t *dp_interface;

	memset(&globals, 0, sizeof(globals));

	switch_queue_create(&globals.queue, DB_QUEUE_LEN, pool);

	switch_mutex_init(&globals.mutex, SWITCH_MUTEX_NESTED, pool);

	globals.pool = pool;

	load_config();

	*module_interface = switch_loadable_module_create_module_interface(pool, modname);

	SWITCH_ADD_APP(app_interface, DEMO1_APP, "Execute Demo1", "Execute Demo1", demo1_app_function, DEMO1_APP_SYNTAX,
                                   SAF_SUPPORT_NOMEDIA);
	SWITCH_ADD_API(api_interface, DEMO2_API, "DEMO2", demo2_api_function, DEMO2_API_SYNTAX);
	SWITCH_ADD_DIALPLAN(dp_interface, "demo1", demo1_dialplan_hunt);
	// SWITCH_ADD_DIALPLAN(dp_interface, "demo2", demo2_dialplan_hunt);

	/* indicate that the module should continue to be loaded */
	return SWITCH_STATUS_SUCCESS;
}

SWITCH_MODULE_SHUTDOWN_FUNCTION(mod_diydp_shutdown)
{
	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "mod_diydp shutdown\n");
	return SWITCH_STATUS_SUCCESS;
}

/* For Emacs:
 * Local Variables:
.* mode:c
.* indent-tabs-mode:t
 * tab-width:4
 * c-basic-offset:4
 * End:
 * For VIM:
 * vim:set softtabstop=4 shiftwidth=4 tabstop=4 expandtab:
 */
